package scrabblePackage;
import java.util.ArrayList;
import java.util.Random;
import java.util.Set;

public class PlayerActions {
	public static int wordScore;
	public static int wordMultiplier;

	public static void getTiles(int numOfTiles) {
		ArrayList<String> rack = Scrabble.getCurrentRack();
		while (numOfTiles > 0) {
			getTile(rack);
			numOfTiles--;
		}
	}


	public static void passTurn() {
		if (Scrabble.exchangeButton != null)
			Scrabble.exchangeButton.setVisible(true);
		Scrabble.switchPlayer();
		Scrabble.clearTurnHistory();
	}

	public static int calculateWordScore() {
		return wordMultiplier * wordScore;
	}

	public static void placeTile(int x, int y) {
		Board.addLetter(x, y);

		Scrabble.lettersOnTurn[x][y] = Scrabble.nextLetter;

		calculateTileScore(x, y);
		Scrabble.visitedDown[x][y] = true;
		Scrabble.visitedAcross[x][y] = true;
		checkFourDirections(x, y);
	}
	
	public static void challengeWord() {
		Scrabble.returnTilesToRack();
		Scrabble.getCurrentPlayer().displayTiles();
		Scrabble.exchangeButton.setVisible(true);
		PlayerActions.wordScore = 0;
		passTurn();
	}

	public static void endTurn() {
		int returnValue = isValidTurn();
		if (Scrabble.needToChallenge == false) {
			int finalWordScore = PlayerActions.calculateWordScore();
			int tilesMissing = 7 - Scrabble.getCurrentRack().size();
			if (tilesMissing == 7 && Scrabble.getNumberOfTilesLeft() > 0) {
				finalWordScore += 50;
			}
			Scrabble.getCurrentPlayer().increasePointTotal(finalWordScore);
			PlayerActions.getTiles(tilesMissing);
			Scrabble.getCurrentPlayer().displayTiles();
			PlayerActions.passTurn();
			return;
		}
		else if (returnValue == 1) { // Valid Turn
			ChallengeFrame f = new ChallengeFrame(Scrabble.scrabble.getBounds());
			f.setVisible(true);
		} else if (returnValue == 2) { // no tiles were played
			passTurn();
			Scrabble.clearTurnHistory();
		} else { // not a valid turn
			NotAValidTurnFrame n = new NotAValidTurnFrame(Scrabble.scrabble.getBounds());
			n.setVisible(true);
			Scrabble.returnTilesToRack();
			Scrabble.getCurrentPlayer().displayTiles();
			Scrabble.clearTurnHistory();
			Scrabble.exchangeButton.setVisible(true);
		}
	}
	
	private static void getTile(ArrayList<String> rack) {
		Random r = new Random();
		Set<String> possibleKeys = Scrabble.letterBag.keySet();
		ArrayList<String> allLettersInBag = new ArrayList<String>();
		for (String myKey : possibleKeys) {
			int amountToAdd = Scrabble.letterBag.get(myKey).numOfTiles;
			for (int index = 0; index < amountToAdd; index++) {
				allLettersInBag.add(myKey);
			}
		}
		if (allLettersInBag.size() == 0)
			return;
		int keyIndex = r.nextInt(allLettersInBag.size());
		String letter = allLettersInBag.get(keyIndex);
		Letters theLetterObject = Scrabble.letterBag.get(letter);
		if (theLetterObject.numOfTiles > 0) {
			rack.add(letter);
			theLetterObject.numOfTiles--;
		} else
			getTile(rack);
	}

	private static void calculateTileScore(int x, int y) {
		String letterOnSquare = Scrabble.activeLetters[x][y];
		if (letterOnSquare.contains("'")) {
			letterOnSquare = "_";
		}
		int letterValue = Scrabble.letterBag.get(letterOnSquare).tileValue;
		int letterMultiplier = Board.playingField[x][y].getLetterMultiplier();
		wordScore += letterValue * letterMultiplier;
		wordMultiplier *= Board.playingField[x][y].getWordMultiplier();
	}

	private static void checkFourDirections(int x, int y) {
		checkDirection(x - 1, y, "left");
		checkDirection(x, y - 1, "up");
		checkDirection(x + 1, y, "right");
		checkDirection(x, y + 1, "down");
	}

	private static void checkDirection(int x, int y, String direction) {
		if (x >= Scrabble.boardSize || x < 0 || y >= Scrabble.boardSize || y < 0)
			return;

		String letterOnSquare = Scrabble.activeLetters[x][y];
		if (letterOnSquare.equals(""))
			return;
		if (hasNotBeenVisited(x, y, direction)) {
			if (letterOnSquare.contains("(")) {
				letterOnSquare = "_";
			}
			int letterValue = Scrabble.letterBag.get(letterOnSquare).tileValue;
			wordScore += letterValue;
		}

		int nextX = x;
		int nextY = y;
		if (direction.equals("left")) {
			Scrabble.visitedAcross[x][y] = true;
			nextX--;
		} else if (direction.equals("up")) {
			Scrabble.visitedDown[x][y] = true;
			nextY--;
		} else if (direction.equals("right")) {
			Scrabble.visitedAcross[x][y] = true;
			nextX++;
		} else if (direction.equals("down")) {
			Scrabble.visitedDown[x][y] = true;
			nextY++;
		}
		checkDirection(nextX, nextY, direction);
	}

	private static boolean hasNotBeenVisited(int x, int y, String direction) {
		return ((direction.equals("up") || direction.equals("down")) && !Scrabble.visitedDown[x][y]) || ((direction.equals("left") || direction.equals("right")) && !Scrabble.visitedAcross[x][y]);
	}
	
	private static int isValidTurn() {
		boolean surroundedByExistingLetter = false;
		boolean noTilesPlayed = true;
		for (int c = 0; c < Scrabble.boardSize; c++) {
			for (int r = 0; r < Scrabble.boardSize; r++) {
				String letterInQuestion = Scrabble.lettersOnTurn[c][r];
				if (!letterInQuestion.equals("")) {
					noTilesPlayed = false;
					if (!surroundedByLetter(c, r))
						return 0;
					if (surroundedByExistingLetter(c, r))
						surroundedByExistingLetter = true;
				}
			}
		}
		if (Scrabble.isFirstTurn) {
			surroundedByExistingLetter = !Scrabble.lettersOnTurn[Math.round(Scrabble.boardSize / 2)][Math.round(Scrabble.boardSize / 2)].equals("");
			Scrabble.isFirstTurn = false;
		}
		if (noTilesPlayed)
			return 2;
		else if (surroundedByExistingLetter)
			return 1;
		else
			return 0;
	}

	private static boolean surroundedByLetter(int x, int y) {
		boolean isSurrounded = false;
		if (x - 1 > 0 && !Scrabble.activeLetters[x - 1][y].equals(""))
			isSurrounded = true;
		else if (y - 1 > 0 && !Scrabble.activeLetters[x][y - 1].equals(""))
			isSurrounded = true;
		else if (x + 1 < Scrabble.boardSize && !Scrabble.activeLetters[x + 1][y].equals(""))
			isSurrounded = true;
		else if (y + 1 < Scrabble.boardSize && !Scrabble.activeLetters[x][y + 1].equals(""))
			isSurrounded = true;
		return isSurrounded;
	}

	private static boolean surroundedByExistingLetter(int x, int y) {
		boolean isSurrounded = false;
		if (x - 1 > 0 && !Scrabble.activeLetters[x - 1][y].equals("") && Scrabble.lettersOnTurn[x - 1][y].equals(""))
			isSurrounded = true;
		else if (y - 1 > 0 && !Scrabble.activeLetters[x][y - 1].equals("") && Scrabble.lettersOnTurn[x][y - 1].equals(""))
			isSurrounded = true;
		else if (x + 1 < Scrabble.boardSize && !Scrabble.activeLetters[x + 1][y].equals("") && Scrabble.lettersOnTurn[x + 1][y].equals(""))
			isSurrounded = true;
		else if (y + 1 < Scrabble.boardSize && !Scrabble.activeLetters[x][y + 1].equals("") && Scrabble.lettersOnTurn[x][y + 1].equals(""))
			isSurrounded = true;
		return isSurrounded;
	}
}
